﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using stepnc_plugin;

namespace SamplePlugin
{
    public class MTConnectPlugin : ICNCPlugin
    {
	const string mtserver = "http://okuma-matata:5000/current";
	const string mtserver2 = "http://localhost:5000/current";
	const string mtconfig = "http://localhost:5000/probe";
	const string mtfile = "file:///C:/Dave/current.xml";

	bool f_connected = false;
	string f_machinename = null;

	public bool LoadCNC()
	{
	    long blk;
	    double x, y, z;
	    // just try to query the server to see if we have a connection
	    f_connected = GetProgramPosition(out blk, out x, out y, out z);

	    // could set the machine name here
	    return f_connected;
	}

	public void UnloadCNC()
	{
	    // nothing needed
	}

	public bool IsConnected()
	{
	    return f_connected;
	}

	public string GetMachineDesc()
	{
	    return f_machinename;
	}

	public bool GetProgramPosition(out long block, out double x, out double y, out double z)
	{
	    XmlDocument mt = new XmlDocument();
	    block = 0; x = y = z = 0;

	    try {
		// The MT Connect format declares a default XML namespace.  Even
		// though the tags are not prefixed with anything in the file, the
		// DOM calls want an explicit namespace, which we just call "foo".
		// This may just be a .NET thing.
		//
		mt.Load(mtserver);
		XmlNode root = mt.DocumentElement;
		XmlNamespaceManager nsmgr = new XmlNamespaceManager(mt.NameTable);
		nsmgr.AddNamespace("foo", root.NamespaceURI);
		
		// Cache the machine name on the first call.  We get this from
		// the name attribute of the first <DeviceStream> that we see.
		if (f_machinename == null) {
		    XmlNode n = root.SelectSingleNode("//foo:DeviceStream", nsmgr);
		    f_machinename = n.Attributes.GetNamedItem("name").InnerText;
		}

		// Get the XYZ position from the first <PathPosition> element
		// that we find.  The contents are a space separated list of
		// coordinates.
		XmlNode p = root.SelectSingleNode("//foo:PathPosition", nsmgr);
		string trip = p.InnerText;
		var parts = trip.Split(' ');
		try {
		    x = Convert.ToDouble(parts[0]);
		    y = Convert.ToDouble(parts[1]);
		    z = Convert.ToDouble(parts[2]);
		}
		catch (Exception)
		{
		    x = y = z = 0;
		}

		// Get the Block position from the first <Line> element
		// that we find.  
		p = root.SelectSingleNode("//foo:Line", nsmgr);
		string blkstr = p.InnerText;
		try {
		    block = Convert.ToInt64(blkstr.TrimStart(' ', 'N'));
		}
		catch (Exception) {
		    block = 0;
		}
		return true;
	    }
	    catch(Exception) {
		return false;
	    }

	}

	public UnitType GetProgramPositionUnit()
	{
	    return UnitType.UnitMM;
	}

	public BlockNumberType GetProgramBlockType()
	{
	    // For now we are assuming that MTConnect is always going to return
 	    // the N123 label on a G-code block.
	    return BlockNumberType.NLabel;
	}


	public bool IsSTEPNCPosition()
	{
	    return true;
	}


	public void SetCurrentProgram(string path)
	{
	    // not supported
	}

    }
}
