﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Net;
using System.IO;

using stepnc_plugin;

namespace SamplePlugin
{
    public class TextPlugin : ICNCPlugin
    {
	const string mtserver = "http://localhost:8080/getMachineLocation";

	bool f_connected = false;
	string f_machinename = null;
	UnitType f_unit = UnitType.UnitMM;

	public bool LoadCNC()
	{
	    long blk;
	    double x, y, z;
	    // just try to query the server to see if we have a connection
	    f_connected = GetProgramPosition(out blk, out x, out y, out z);

	    // could set the machine name here
	    return f_connected;
	}

	public void UnloadCNC()
	{
	    // nothing needed
	}

	public bool IsConnected()
	{
	    return f_connected;
	}

	public string GetMachineDesc()
	{
	    return f_machinename;
	}

	public bool GetProgramPosition(out long block, out double x, out double y, out double z)
	{
	    block = 0; x = y = z = 0;
	    WebRequest mt = WebRequest.Create(mtserver);
	    try
	    {

		Stream strm = mt.GetResponse().GetResponseStream();
		StreamReader objReader = new StreamReader(strm);

		// Read a line and extract bar-delimited fields
		string sLine = sLine = objReader.ReadLine();
		if (sLine == null) return false;

		// Get the XYZ position.  The contents are a space separated list of
		// coordinates.
		var parts = sLine.Split('|');
		f_machinename = parts[0];

		if (parts[1] == "INCHES") f_unit = UnitType.UnitInch;

		x = Convert.ToDouble(parts[2]);
		y = Convert.ToDouble(parts[3]);
		z = Convert.ToDouble(parts[4]);
		
		// Skip initial N on the block number
		block = Convert.ToInt64(parts[9].TrimStart(' ', 'N'));

		return true;
	    }
	    catch (Exception)
	    {
		return false;
	    }
	}

	public UnitType GetProgramPositionUnit()
	{
	    return f_unit;
	}

	public BlockNumberType GetProgramBlockType()
	{
	    // For now we are assuming that MTConnect is always going to return
 	    // the N123 label on a G-code block.
	    return BlockNumberType.NLabel;
	}


	public bool IsSTEPNCPosition()
	{
	    return true;
	}


	public void SetCurrentProgram(string path)
	{
	    // not supported
	}

    }
}
